/*-
 * Copyright (C)2021 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2021 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: pr.c,v 1.2 2021/03/13 20:18:27 babolo Exp $"

#define BLIN_COMPAT     3

#include <sys/types.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <qrcodegen/qrcodegen.h>
#include <babolo/BLINflag.h>
#include "qrben.h"

int
/*****************************************************************************************************
 *****************************************************************************************************
 ****  PBM  QR   stdout                                                           ****
 *****************************************************************************************************
                                                                                                  ****/
qrben_pbm(qrben_conf *cf, qrben_code *qr, FILE *img) {                                           /****
                                                                                                  ****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int      ex = EX_OK;
    u_int    bs = ((QRBEN_BORDER & cf->flags) ? 8 : 0) + qr->size;

    fprintf(img, "P1\n%u %u\n", bs, bs);
    if  (QRBEN_BORDER & cf->flags) for (u_int i = 0; i < 4; ++i) {
        for (u_int j = 0; j < bs; ++j) fprintf(img, "0");
        fprintf(img, "\n");
    }
    for (u_int i = 0; i < qr->size; ++i) {
        if  (QRBEN_BORDER & cf->flags) fprintf(img, "0000");
        fprintf(img, "%.*s", qr->size, &qr->code[qr->size * i]);
        if  (QRBEN_BORDER & cf->flags) fprintf(img, "0000");
        fprintf(img, "\n");
    }
    if  (QRBEN_BORDER & cf->flags) for (u_int i = 0; i < 4; ++i) {
        for (u_int j = 0; j < bs; ++j) fprintf(img, "0");
        fprintf(img, "\n");
    }
    return(ex);
#   undef blin_internal_flags
}

int
/*****************************************************************************************************
 *****************************************************************************************************
 ****  PBM  QR   stdout                                                  ****
 *****************************************************************************************************
                                                                                                  ****/
qrben_pbmsz(qrben_conf *cf, qrben_code *qr, FILE *img) {                                         /****
                                                                                                  ****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int      ex = EX_OK;
    u_int    bs = ((QRBEN_BORDER & cf->flags) ? 8 : 0) + qr->size;

    fprintf(img, "P1\n%u %u\n", bs * cf->x, bs * cf->y);
    if  (QRBEN_BORDER & cf->flags) for (u_int i = 0; i < (4 * cf->y); ++i) {
        for (u_int j = 0; j < (bs * cf->x); ++j) fprintf(img, "0");
        fprintf(img, "\n");
    }
    for (u_int i = 0; i < qr->size; ++i) {
        for (u_int y = 0; y < cf->y; ++y) {
            if  (QRBEN_BORDER & cf->flags) for (u_int j = 0; j < (4 * cf->x); ++j) fprintf(img, "0");
            for (u_int j = 0; j < qr->size; ++j) {
                for (u_int x = 0; x < cf->x; ++x) {
                    fprintf(img, "%c", (qr->code[qr->size * i + j] & 1) ? '1' : '0');
            }   }
            if  (QRBEN_BORDER & cf->flags) for (u_int j = 0; j < (4 * cf->x); ++j) fprintf(img, "0");
            fprintf(img, "\n");
    }   }
    if  (QRBEN_BORDER & cf->flags) for (u_int i = 0; i < (4 * cf->y); ++i) {
        for (u_int j = 0; j < (bs * cf->x); ++j) fprintf(img, "0");
        fprintf(img, "\n");
    }
    return(ex);
#   undef blin_internal_flags
}

int
/*****************************************************************************************************
 *****************************************************************************************************
 ****  PBM  QR   stdout                                                  ****
 *****************************************************************************************************
                                                                                                  ****/
qrben_show(qrben_conf *cf, qrben_code *qr) {                                                      /****
                                                                                                  ****
 *****************************************************************************************************
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int      ex = EX_OK;

    for (int i = 0; i < qr->size; ++i) {
        for (u_int y = 0; y < cf->y; ++y) {
            for (int j = 0; j < qr->size; ++j) {
                for (u_int x = 0; x < cf->x; ++x) {
                    printf("%c", (qr->code[qr->size * i + j] & 1) ? '#' : ' ');
            }   }
            printf("\n");
    }   }
    return(ex);
#   undef blin_internal_flags
}
