/*-
 * Copyright (C)2021 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2021 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: qrben.c,v 1.2 2021/03/13 20:18:27 babolo Exp $"

#define BLIN_COMPAT     3
#define MIFE_COMPAT     5
#define Bpars_COMPAT    4

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <qrcodegen/qrcodegen.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <mife.h>
#include "qrben.h"
#include "md.h"

static int
/*****************************************************************************************************
                                                                                                  ****/
testalfa(qrben_conf *cf) {                                                                       /****
                                                                                                  ****
 *****************************************************************************************************/
#   define blin_internal_flags (cf->flags)
    int ex = EX_OK;
    u_int t;

    ifBLIN_QX3("+ %08X", cf->flags);
    t = 0;
    for (u_int i = 0; i < 128; ++i) {
        if  ((qrcodegen_Mode_NUMERIC == qlass[i]) || (qrcodegen_Mode_ALPHANUMERIC == qlass[i])) ++t;
    }
    printf("alpha %d, qlass # %d\n", (int)strlen(qrben_alpha), t);
    if  (strlen(qrben_alpha) != t) {
        ifBLIN_QX0("alpha <> qlass #");
        ++ex;
    }
    for (u_int i = 0; i < strlen(qrben_alpha); ++i) {
        if  (  (qrcodegen_Mode_NUMERIC != qlass[qrben_alpha[i] & 0x0FF])
            && (qrcodegen_Mode_ALPHANUMERIC != qlass[qrben_alpha[i] & 0x0FF])
            ) {
            ifBLIN_QX0("%02X(%c)=%u", qrben_alpha[i], qrben_alpha[i], qlass[qrben_alpha[i] & 0x0FF]);
            ex = 1;
    }   }
    for (u_int i = 0; i < 128; ++i) {
        if  ((qrcodegen_Mode_ALPHANUMERIC == qlass[i]) && !index(qrben_alpha, i)) {
            ifBLIN_QX0("%02X(%c)Not in", i, i);
            ex = 1;
    }   }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

static void
/*****************************************************************************************************
                                                                                                  ****/
usage(int ex) {                                                                                  /****
                                                                                                  ****
 *****************************************************************************************************/
    fprintf( stderr
           , PKG" @BABOLO V.M "VERS" http://www.babolo.ru/\n"
             "Usage: "PKG" [-options][ text]\n"
             "   -?            This text\n"
           )
    ;
    exit(ex);
}

int
/*****************************************************************************************************
 *****************************************************************************************************
                                                                                                  ****/
main(int argc, char **argv) {                                                                    /****
                                                                                                  ****
 *****************************************************************************************************
 *****************************************************************************************************/
    const char       *flar   = "a:bci:p:qs:S:t::u:U:vx:y:=:?:";
    const char       *Tflag  = NULL;
    int               ex     = EX_OK;
    qrben_code       *qrcode = NULL;
    babolo_opts      *bos;
    qrben_conf       *cf;
    const char       *o;
    int               c;

    QRBEN_W(!(cf = calloc(1, sizeof(qrben_conf))),                           ("qrben_conf"), EX_OSERR);
    blin_ctl(BLIN_CTL_FSET, (cf->flags = BLIN_BIT0));
#   define blin_internal_flags (cf->flags)
    QRBEN_W(!(bos = babolo_openopts(Bpars_ESYM | Bpars_SYME, 'Z')),         ("openopts"), EX_SOFTWARE);
    QRBEN_W(babolo_setopts(bos, Bpars_NONU, argc, argv, flar),               ("setopts"), EX_SOFTWARE);
    cf->versize = qrcodegen_VERSION_MAX;
    cf->x = 1;
    cf->y = 1;
    for (; !!(c = babolo_getopts(bos));) {
        ifBLIN_QX4("-%02X", c);
        QRBEN_W((0 > c),                                                 ("babolo_getopts"), EX_USAGE);
        switch (c) {
        case 'a': QRBEN_W(!(o = babolo_getoptsarg(bos)),       ("No -%c arg 2", bos->c), EX_USAGE);
                ; QRBEN_O(!!(ex = qrben_aseg(cf, strlen(o), o, cf->versize)),                ("aseg"));
                ; break;
        case 'b': cf->flags |= QRBEN_BORDER;
                ; break;
        case 'c': cf->flags |= QRBEN_SHOW;
                ; break;
        case 'i': QRBEN_W(!(o = babolo_getoptsarg(bos)),             ("No -%c arg", bos->c), EX_USAGE);
                ; cf->imgname = strdup(o);
                ; break;
        case 'p': QRBEN_W(!(o = babolo_getoptsarg(bos)),       ("No -%c arg 2", bos->c), EX_USAGE);
                ; QRBEN_O(!!(ex = qrben_pseg(cf, strlen(o), o, cf->versize)),                ("pseg"));
                ; break;
        case 'q': BLIN_QUIET(cf->flags);
                ; break;
        case 's': cf->versize = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; cf->flags |= QRBEN_FIXSIZE;
                ; break;
        case 'S': cf->versize = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; cf->flags &= ~QRBEN_FIXSIZE;
                ; break;
        case 't': {   qrben_mode mode;

                ;     QRBEN_W(!(o = babolo_getoptsarg(bos)),       ("No -%c arg 1", bos->c), EX_USAGE);
                ;     QRBEN_W(!(mode = babolo_testword(&md, o)),         ("Ill mode %s", o), EX_USAGE);
                ;     QRBEN_W(!(o = babolo_getoptsarg(bos)),       ("No -%c arg 2", bos->c), EX_USAGE);
                ;     QRBEN_O(!!(ex = qrben_addseg(cf, strlen(o), o, mode)),               ("addseg"));
                ; }
                ; break;
        case 'u': cf->errc = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; cf->flags &= ~QRBEN_FIXERRC;
                ; break;
        case 'U': cf->errc = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; cf->flags |= QRBEN_FIXERRC;
                ; break;
        case 'v': BLIN_VERBOSE(cf->flags);
                ; break;
        case 'x': cf->x = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; break;
        case 'y': cf->y = babolo_getoptunum(bos, 0);
                ; QRBEN_W((!!errno),                             ("getoptunum -%c", bos->c), EX_USAGE);
                ; break;
        case '=': blin_ctl(BLIN_CTL_FEQU, cf->flags);
                ; switch((Tflag = babolo_getoptsarg(bos))[0]) {
                  case '0': blin_ctl(BLIN_CTL_DUMP);     exit(EX_OK);
                  case '1': babolo_optest(flar);         exit(EX_OK);
                  case '2': babolo_optext(bos);          exit(EX_OK);
                  case '3': babolo_dumpopts(bos);        exit(EX_OK);
                  case '4': exit(testalfa(cf));
                  default : usage(EX_USAGE);
                ; }
                ; exit(ex);
        case '?': usage(EX_OK);
        default : ifBLIN_QX0("Illegal flag -%c", bos->c);
                ; usage(EX_USAGE);
    }   }
    QRBEN_W(!(qrcode = qrben_gen(cf)),             ("gen"), (errno = ENOMEM) ? EX_OSERR : EX_SOFTWARE);
    if  (QRBEN_SHOW & cf->flags) QRBEN_O(!!(ex = qrben_show(cf, qrcode)),                    ("show"));
    if  (!!cf->imgname) {
        FILE *img;

        QRBEN_W(!(img = fopen(cf->imgname, "w")),                 ("fopen %s", cf->imgname), EX_IOERR);
        if  ((1 == cf->x) && (1 == cf->y)) {
            QRBEN_O(!!(ex = qrben_pbm(cf, qrcode, img)),                                      ("pbm"));
        } else {
            QRBEN_O(!!(ex = qrben_pbmsz(cf, qrcode, img)),                                  ("pbmsz"));
    }   }
    babolo_closeopts(bos);
    free(qrcode);
    for (int i = 0; i < cf->nseg; ++i) free(cf->segs[i].data);
    free(cf->imgname);
    free(cf->segs);
    free(cf);
out:
    exit(ex);
}
